@@ -17,7 +17,7 @@ $apt_cache $apt_source $home $host_config $mirror $verbose @dirs
 deplookup force_update check_update setup_config set_cachedir
 update_sources prepare_sources_list get_aptcross_dir init_cache
 get_verbose set_mirror set_verbose init_host_cache ver_compare 
-clear_config );
+clear_config update_multiarch );
 
 =pod
 
@@ -75,7 +75,6 @@ Includes code from apt-rdepends to get the Depends info efficiently.
 
 =cut
 
-$mirror = '';
 # from apt-rdepends
 #
 # apt-rdepends has a wider support for dependencies than
@@ -117,6 +116,7 @@ sub aptinit {
 	$_config->set("Apt::Architecture", $config->{'main'}->{'arch'})
 		if($config->{'main'}->{'arch'} ne '');
 	$_config->set("APT::Get::List-Cleanup", "off");
+	$_config->set("APT::Get::AllowUnauthenticated", "true");
 	$_config->set("Dir", $config->{'main'}->{'maindir'});
 	$_config->set("Dir::Etc", $config->{'main'}->{'maindir'});
 	$_config->set("Dir::Etc::SourceList", &Cache::Apt::Config::get_sourceslist);
@@ -171,7 +171,7 @@ sub init_host_cache
 	$verbose = shift;
 	return ($host_config) if (defined($host_config));
 	$apt_cross_dir = get_aptcross_dir();
-	$suite = Cache::Apt::Config::get_suite();
+	$suite = &Cache::Apt::Config::get_suite();
 	$host_config = new Cache::Apt::Config(
 		maindir => $apt_cross_dir,
 		suite => $suite,
@@ -200,7 +200,7 @@ sub init_cache
 	$verbose = shift;
 	return ($config) if (defined($config));
 	$apt_cross_dir = get_aptcross_dir();
-	$suite = Cache::Apt::Config::get_suite();
+	$suite = &Cache::Apt::Config::get_suite();
 	$arch = &Cache::Apt::Config::get_cache_architecture();
 	$config = new Cache::Apt::Config(
 		maindir => $apt_cross_dir,
@@ -420,7 +420,7 @@ sub force_update
 	$arch = &Cache::Apt::Config::get_cache_architecture();
 	$suite = &Cache::Apt::Config::get_suite();
 	$verbose = &get_verbose;
-	if ($mirror) {
+	if (defined $mirror) {
 		print "Updating $suite on $arch using $mirror\n" if $verbose >= 2;
 	}
 	else {
@@ -430,8 +430,12 @@ sub force_update
 	my $mysrc = &Cache::Apt::Config::check_mysources;
 	unlink ("$apt_cross_dir/sources.$suite") unless defined($mysrc);
 	unlink ("$apt_cross_dir/.aptupdate-stamp-$suite-$arch");
+	if (not -e "$apt_cross_dir/preferences.d") {
+		mkdir "$apt_cross_dir/preferences.d";
+	}
 	&setup_config;
 	&update_sources;
+	&update_multiarch;
 }
 
 =head1 check_update
@@ -446,8 +450,8 @@ sub check_update
 {
 	$verbose = shift;
 	$arch = &Cache::Apt::Config::get_cache_architecture();
-	$suite = Cache::Apt::Config::get_suite();
-	if ($mirror) {
+	$suite = &Cache::Apt::Config::get_suite();
+	if (defined $mirror) {
 		print "Checking $suite on $arch using $mirror\n" if $verbose >= 2;
 	}
 	else {
@@ -469,7 +473,7 @@ using update_sources.
 
 sub setup_config
 {
-	$suite = Cache::Apt::Config::get_suite() if (not defined ($suite));
+	$suite = &Cache::Apt::Config::get_suite() if (not defined ($suite));
 	$arch = &Cache::Apt::Config::get_cache_architecture();
 	$apt_cross_dir = &get_aptcross_dir();
 	@dirs = qw/ alternatives info parts updates/;
@@ -481,6 +485,9 @@ sub setup_config
 	if (not -e "$apt_cross_dir/$suite/lists") {
 		mkdir "$apt_cross_dir/$suite/lists";
 	}
+	if (not -e "$apt_cross_dir/preferences.d") {
+		mkdir "$apt_cross_dir/preferences.d";
+	}
 	if (not -e "$apt_cross_dir/$suite/lists/partial") {
 		mkdir "$apt_cross_dir/$suite/lists/partial";
 	}
@@ -505,12 +512,12 @@ sub setup_config
 	my $mysrc = &Cache::Apt::Config::check_mysources;
 	return if (defined $mysrc);
 	&prepare_sources_list;
-	if ((-f "$apt_cross_dir/sources.$suite") && ($mirror ne "")) {
+	if ((-f "$apt_cross_dir/sources.$suite") && (defined $mirror)) {
 		open (SOURCES, "$apt_cross_dir/sources.$suite") || die "Cannot open apt-cross sources list for $suite $!";
 		my @contents=<SOURCES>;
 		my $check = join (@contents);
 		close (SOURCES);
-		if ((!$check =~ /$mirror/) &&($mirror)) {
+		if (!$check =~ /$mirror/) {
 			# if it is missing, append it.
 			open (SOURCES, ">>$apt_cross_dir/sources.$suite") ||
 				die "Cannot open apt-cross sources list for $suite $!";
@@ -531,7 +538,7 @@ END
 	if (! -f "$apt_cross_dir/sources.$suite")
 	{ #file is missing, so create new.
 		print "debug: recreating $apt_cross_dir/sources.$suite\n" if $verbose >= 2;
-		if ($mirror) {
+		if (defined $mirror) {
 			print "Adding $mirror to $apt_cross_dir/sources.$suite\n" if $verbose >= 2;
 			open (SOURCES, ">>$apt_cross_dir/sources.$suite") || die "Cannot open sources.$suite: $!";
 			print SOURCES<<END;
@@ -545,8 +552,7 @@ END
 		{
 			next if $source =~ /^#/;
 			next if $source =~ /^$/;
-			$source =~ s/deb (.*) (.*) main(.*)/deb $1 $suite main${3}/;
-			$source =~ s/deb-src (.*) (.*) main(.*)/deb-src $1 $suite main${3}/;
+			next if ((defined $suite) and ($source !~ /$suite/));
 			print SOURCES $source;
 		}
 		close SOURCES;
@@ -603,14 +609,16 @@ sub cache_update {
 		print "cache_update: " . Dumper($self) if ($verbose >=4);
 		system(" apt-get ".$self->{main}->{'config_str'}." update 2>/dev/null")
 			if ($verbose >= 2);
-		system(" apt-get ".$self->{main}->{'config_str'}." update &>/dev/null")
+		my $str = $self->{main}->{'config_str'}; 
+		`apt-get $str update 2>/dev/null`
 			if ($verbose < 2);
 	}
 	&aptinit($self, $verbose);
 	return unless defined($host_config);
 	system(" apt-get ".$host_config->{main}->{'config_str'}." update 2>/dev/null")
 		if ($verbose >= 2);
-	system(" apt-get ".$host_config->{main}->{'config_str'}." update &>/dev/null")
+	my $str = $self->{main}->{'config_str'}; 
+	`apt-get $str update 2>/dev/null`
 		if ($verbose < 2);
 	&aptinit($host_config, $verbose);
 }
@@ -645,6 +653,7 @@ sub prepare_sources_list
 		}
 		closedir (FILES);
 	}
+	return \@source_list;
 }
 
 =head1 get_provides
@@ -656,7 +665,7 @@ provided, not what the specified package provides, so this
 routine iterates over the cache and creates a suitable
 index. Subsequent queries are made against the new hash.
 
-e.g. When quering apt-cache, you may expect to see:
+e.g. When querying apt-cache, you may expect to see:
  $ apt-cache show cdebconf | grep Provides
  Provides: debconf-2.0
 
@@ -698,6 +707,76 @@ sub get_provides {
 	return $provides{$pkg};
 }
 
+=head1 update_multiarch
+
+Calculates the Packages file for the requested suite and architecture
+and uses that to update the multiarch lists. Warns if the calculated
+file does not exist - in which case, use the C<--suite> or C<-S> options
+to F<apt-cross> to explicitly state which suite to use. (This can happen
+if you use codenames in your F</etc/apt/sources.list> rather than
+suite names (sid instead of unstable). Apart from sid, the mapping from
+a codename to a suite name changes with each release and this module
+does not attempt to keep up with these changes.
+
+Once found, the routine checks for 
+F</usr/share/apt-cross/update-multiarch.pl> and passes the Packages
+file to it.
+
+=cut
+
+sub update_multiarch
+{
+	my ($repo, $sname);
+	$arch = &Cache::Apt::Config::get_cache_architecture();
+	$suite = &Cache::Apt::Config::get_suite();
+	$verbose = &get_verbose;
+	$apt_cross_dir = get_aptcross_dir();
+	my $mysrc = &prepare_sources_list;
+	$sname = $suite;
+	$sname = "sid" if ($suite eq "unstable");
+	foreach my $src (@$mysrc)
+	{
+		chomp $src;
+		next if ($src =~ /^#/);
+		next if ($src eq "");
+		next if ($src =~ /^deb-src/);
+		if ($src =~ /$suite/ or $src =~ /$sname/)
+		{
+			$repo = $src;
+			$repo =~ m#^deb (f|ht)tp://([^ ]*) ([^ ]*) *([a-z\-]+) *([a-z\-]*) *([a-z\-]*)$#;
+			my $rep = $2;
+			my $cmpnt = $4;
+			$rep =~ s:/$::;
+			$rep =~ s#/#_#g;
+			$repo = $apt_cross_dir . "/" . $suite . "/lists/" . $rep .
+				"_dists_${suite}_${cmpnt}_binary-${arch}_Packages";
+			$repo = $apt_cross_dir . "/" . $suite . "/lists/" . $rep .
+				"_dists_${sname}_${cmpnt}_binary-${arch}_Packages"
+				if (not -f $repo);
+			last if (-f $repo);
+		}
+	}
+	if (not defined $repo)
+	{
+		warn "Unable to calculate the correct Packages file to update multiarch!\n";
+		print "Try specifying a suite explicitly with --suite\n";
+		return;
+	}
+	if (not -f $repo)
+	{
+		warn "Unable to calculate the correct Packages file to update multiarch!\n$repo\n";
+		print "Try specifying a suite explicitly with --suite\n";
+		return;
+	}
+	elsif ((-x "/usr/share/apt-cross/update-multiarch.pl") and
+		(defined $repo))
+	{
+		print "Updating multiarch config using:\n  '$repo'\n"
+			if ($verbose >= 2);
+		system ("/usr/share/apt-cross/update-multiarch.pl -f $repo");
+	}
+}
+
 ### (End of public functions) ###
 
 sub this_cache { return $apt_cache }
