@@ -119,7 +119,8 @@ AptCrossDependency instances describing the
 dependency.
 
 VersionLimit is the version indicator for this dependency and the
-version string itself - but not the parentheses.
+version string itself - but not the parentheses. (Undefined in
+the case of OR dependencies.)
 
  e.g.
  >= 1.2.3-1
@@ -131,8 +132,24 @@ version string itself - but not the parentheses.
 	"VersionLimit" => '$',
 	"Installed" => '$',
 	"Type" => '$',
+	"Operator" => '$',
+	"List" => '$',
  });
 
+AptCrossDependency::List contains the OR'd list of packages
+if AptCrossDependency::Operator eq "|" and is undefined otherwise.
+
+'debconf' is special-cased because the '|' operator does not
+appear in the apt-cache data. Incidences of this hack will show
+up in STDERR if $verbose >= 3 but are otherwise silent. The
+hack converts 'debconf' into 'debconf (>= 0.5) | debconf-2.0'
+in AptCrossDependency::List and sets AptCrossDependency::Operator to '|'
+but includes support for specific version limits if those
+are specified, e.g. 'debconf (>= 1.2.0) | debconf-2.0'. If there
+are other packages where the OR'd list is incorrect, please
+check the output from using &output_pkg with $verbose >= 3 and
+include the output in a bug report against libcache-apt-perl. 
+
 =cut
 
 struct (AptCrossDependency => {
@@ -140,6 +157,8 @@ struct (AptCrossDependency => {
 	"VersionLimit" => '$',
 	"Installed" => '$',
 	"Type" => '$',
+	"Operator" => '$',
+	"List" => '$',
 });
 
 sub populate_depends
@@ -153,6 +172,8 @@ sub populate_depends
 		$dep->VersionLimit($$depends{$name}[0]);
 		$dep->Installed($$depends{$name}[1]);
 		$dep->Type($$depends{$name}[2]);
+		$dep->Operator($$depends{$name}[3]);
+		$dep->List($$depends{$name}[4]) if ($$depends{$name}[3] eq "|");
 		push @list, $dep;
 	}
 	$$pkg->Depends(\@list);
@@ -232,17 +253,34 @@ sub output_pkg {
 	my $dep = $$emp->Depends;
 	my $deps;
 	my $line = "";
-	my @a=();
+	my %dependencies=();
+	my %replaces=();
+	my %conflicts=();
 	foreach my $d (@$dep)
 	{
-		$deps = $$d->Type . ":";
-		$line .= $$d->Package . " (" . $$d->VersionLimit . ")" if ($$d->VersionLimit);
+		$line = "";
+		# List is only defined if $$d->Operator eq '|'
+		# and contains the full list of OR'd dependencies.
+		if (defined $$d->List)
+		{
+			$line = $$d->List;
+		}
+		elsif (defined $$d->VersionLimit)
+		{
+			$line = $$d->Package . " (" . $$d->Operator ." ". $$d->VersionLimit .")";
+		}
+		else
+		{
+			$$d->Package;
+		}
 		$line =~ s/\s\s+/ /g;
-		push @a, $line if ($line ne "");
-		undef ($deps) if ($line eq "");
-		$line="";
+		$dependencies{$line}++ if ($line ne "") and ($$d->Type eq "Depends");
+		$replaces{$line}++ if ($line ne "") and ($$d->Type eq "Replaces");
+		$conflicts{$line}++ if ($line ne "") and ($$d->Type eq "Conflicts");
 	}
-	print $deps . join(", ",@a) . "\n" if (defined($deps));
+	print "Depends: " . join(", ",keys %dependencies) . "\n" if (keys %dependencies);
+	print "Replaces: " . join(", ",keys %replaces) . "\n" if (keys %replaces);
+	print "Conflicts: " . join(", ",keys %conflicts) . "\n" if (keys %conflicts);
 	print "Suggests: ".$$emp->Suggests."\n" if ($$emp->Suggests);
 	print "Recommends: ".$$emp->Recommends."\n" if ($$emp->Recommends);
 	print "Priority: " . $$emp->Priority . "\n" if ($$emp->Priority);
@@ -275,6 +313,8 @@ sub get_depends_list
 				$dep->VersionLimit($$g[0]);
 				$dep->Installed($$g[1]);
 				$dep->Type($$g[2]);
+				$dep->Operator($$g[3]);
+				$dep->List($$g[4]) if ($$g[3] eq "|");
 				push @list, \$dep;
 			}
 		}
